//###################################################################################
// File: sramtest.c
//
// Test program for the 128kB external SRAM memory board.
// Only 60kB in two banks are usable because lower 4kB are
// mapped to internal SRAM and registers of the ATMega.
//
// I did not need any waitstates at 16MHz with a 70ns RAM
//
//###################################################################################
// Compiler: AVR-GCC (Mingw32)
// Version : 3.2
//###################################################################################
// Processor: ATMega128
//
// hk@holger-klabunde.de
// www.holger-klabunde.de
// Last change: 13.09.2003
//###################################################################################

#include <io.h>
#include <interrupt.h>
#include <sig-avr.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

#include "mydefs.h"
#include "protos.h"

#define XRAMSTART   RAMEND+1   //start of external RAM = 0x1100 for ATMega128
                               //each RAM bank than has 0xEF00 = 61184 bytes

//use PORTG4 to switch RAM on off
#define ENABLE_RAM()  { cbi(PORTG,4); }
#define DISABLE_RAM() { sbi(PORTG,4); }

//use PORTG3 to select RAM banks
#define RAM_BANK0()  { cbi(PORTG,3); } //lower 60kB
#define RAM_BANK1()  { sbi(PORTG,3); } //upper 60kB

//"volatile" fr alle Variablen die in Interrupt's gendert werden
unsigned int volatile overflow;

//###################################################################################
SIGNAL(SIG_OVERFLOW1)        /* signal handler for tcnt1 overflow interrupt */
//###################################################################################
{
 if(overflow>0) overflow--;
}

//###################################################################################
/* Hauptprogramm */
int main(void)
//###################################################################################
{
 unsigned char *p;         //pointer to RAM
 unsigned char volatile c; //you have to use volatile for c=*p++;
 unsigned char i,err;

 DDRB=0xFF;  	// set io-pins on PortB
 PORTB=0; 	//LCD-Datenbus

 DDRC=0xFF;  	// set io-pins on PortC
 PORTC=0; 	//RAM-Datenbus

 DDRD=0xF0;  	// set io-pins on PortD
 PORTD=0; 	//LCD-Steuersignale siehe lcd.h

 DDRG=0xFF;  	// set io-pins on PortG
 PORTG=0xFF; 	//RAM /WR,/RD,ALE,CE und Bank Select

 MCUCR|=1<<SRE; //Enable external RAM

 LCDInit();
 LCDCls();
 LCDWrite("ATMega128 SRAM-Test");
 LCDPos(2,1);
 LCDWrite("2003 by Holgi");
 Delay1ms(3000);

 ENABLE_RAM();
 for(;;) // Anfang Programm Endlosschleife
  {
   
    err=0;        //if greater 0 means error
   //Write Bank0
    LCDCls();
    LCDWrite("Writing Bank0");
    Delay1ms(1000);
    RAM_BANK0();
    p=(unsigned char *)XRAMSTART;
    i=0;
    do
     {
      *p++=i;
      i++;
     }while(p!=(unsigned char*)0xFFFF);

    *p=i; //Write the last location ;)
   //End Write Bank0 

   //Write Bank1
    RAM_BANK1();
    LCDPos(2,1);
    LCDWrite("Writing Bank1");
    Delay1ms(1000);
    p=(unsigned char *)XRAMSTART;
    i=128; //shift values to test Bank1 is really used and
           //we do not write to Bank0 
    do
     {
      *p++=i;
      i++;
     }while(p!=(unsigned char*)0xFFFF);

    *p=i; //Write the last location ;)
   //End Write Bank1 

   //Verify Bank0
    RAM_BANK0();
    LCDPos(1,15);
    LCDWrite("Verifying");
    Delay1ms(1000);
    p=(unsigned char *)XRAMSTART;
    i=0;
    do
     {
      c=*p++;
      if(c!=i) err=1;
      i++;
     }while(p!=(unsigned char*)0xFFFF);

    //Verify the last location ;)
    c=*p;
    if(c!=i) err=1;
    if(err!=0) { LCDWrite(" RAM Err"); }
    else { LCDWrite(" ok"); } 
    Delay1ms(1000);
   //End Verify Bank0

   //Verify Bank1
    err=0;
    RAM_BANK1();
    LCDPos(2,15);
    LCDWrite("Verifying");
    Delay1ms(1000);
    p=(unsigned char *)XRAMSTART;
    i=128; //shift values to test Bank1 is really used and
           //we do not verify with Bank0 
    do
     {
      c=*p++;
      if(c!=i) err=1;
      i++;
     }while(p!=(unsigned char*)0xFFFF);

    //Verify the last location ;)
    c=*p;
    if(c!=i) err=1;
    if(err!=0) { LCDWrite(" RAM Err"); }
    else { LCDWrite(" ok"); } 
   //End Verify Bank1

   Delay1ms(3000);
  } // Ende Programm Endlosschleife
}

//###################################################################################
//Nicht sonderlich genau !!
void Delay1ms(unsigned int time)
//###################################################################################
{
 unsigned int i;

 for(i=0; i<time; i++) Delay100us(10);
}

//###################################################################################
//Nicht sonderlich genau !!
//time<=255 !!
void Delay100us(unsigned char time)
//###################################################################################
{
 unsigned char k,l;

 for(k=0; k<time; k++)
  {
   for(l=0; l<40; l++)
    {
     //Ein CPU-Clock = 0.125us bei 8MHz
     NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();  
     NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();  
    }
  }
}
